/*
   ZoranIOCTL.c

   ZORAN ZR36120 module driver.
   Copyright (C) 1997  Vincent Labie (try ivsparis@pratique.fr with
				      for Vincent Labie in the suject)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include	"ZoranIO.h"
#include	"SAA7110.h"
#include	"ZoranI2C.h"

/**********************************************************************************************************************/
	int	ZoranIOCTL2(ZORAN * pZoran,unsigned int Command,unsigned long arg)
{
	int			Result	=0;

	ZRDBG(("Zoran%d: ioctl command 0x%08X arg 0x%08lX\n",pZoran->IndexPCI,Command,arg));
	switch(Command)
		{
		case ZORAN_CAP_PARAMS:
			{
			ZR_CAP_PARAMS			ZRCapParams;
			if ((Result=verify_area(VERIFY_READ,(ZR_CAP_PARAMS*)arg,sizeof(ZR_CAP_PARAMS)))!=0) break;
			if ((Result=verify_area(VERIFY_WRITE,(ZR_CAP_PARAMS*)arg,sizeof(ZR_CAP_PARAMS)))!=0) break;
			memcpy_fromfs(&ZRCapParams,(ZR_CAP_PARAMS*)arg,sizeof(ZR_CAP_PARAMS));
			if ((Result=ZoranCapParams(pZoran,&ZRCapParams))!=0) break;
			memcpy_tofs((ZR_CAP_PARAMS*)arg,&ZRCapParams,sizeof(ZR_CAP_PARAMS));
			}
			break;

		case ZORAN_CAPMODE_STOP:
		case ZORAN_CAPMODE_CONTINOUS:
		case ZORAN_CAPMODE_3CONTINOUS:
		case ZORAN_CAPMODE_3CONTINOUSHACK:
		case ZORAN_CAPMODE_GRAB:
			Result=ZoranSetMode(pZoran,Command);
			break;

		case ZORAN_CAP_IS_GRAB:
			Result=ZoranCapIsGrab(pZoran);
			break;
		case ZORAN_CAP_NEW_GRAB:
			Result=ZoranCapNewGrab(pZoran);
			break;

		case ZORAN_CAP_NEW_USER:
			Result=ZoranCapNewUser(pZoran);
			break;

		case ZORAN_CVBS1:
		case ZORAN_CVBS2:
		case ZORAN_CVBS3:
		case ZORAN_CVBS4:
		case ZORAN_CVBS5:
		case ZORAN_CVBS6:
		case ZORAN_Y1C1:
		case ZORAN_Y2C2:
		case ZORAN_Y3C3:
		case ZORAN_CVBS1_NCT:
		case ZORAN_CVBS2_NCT:
		case ZORAN_CVBS3_NCT:
		case ZORAN_CVBS4_NCT:
		case ZORAN_CVBS5_NCT:
		case ZORAN_CVBS6_NCT:
			Result=SAA7110InputSetup(pZoran,Command,arg);
			if (Result>=0)	pZoran->VideoStd=Result&0x0000ffff;
			else		pZoran->VideoStd=0;
			break;

		case ZORAN_HUE:
		case ZORAN_GAIN:
		case ZORAN_SATURATION:
		case ZORAN_CONTRAST:
		case ZORAN_BRIGHTNESS:
			Result=SAA7110Setup(pZoran,Command,arg);
			break;

		case ZORAN_I2C_READ:
			ZRDBG(("ZORAN_I2C_READ1\n"));
			{
			ZR_I2C_ACCESS			ZRI2CAccess;
			u8				TabByte[256];
			if ((Result=verify_area(VERIFY_READ,(ZR_I2C_ACCESS*)arg,
						sizeof(ZR_I2C_ACCESS)))!=0) break;
			memcpy_fromfs(&ZRI2CAccess,(ZR_I2C_ACCESS*)arg,sizeof(ZR_I2C_ACCESS));
			if (ZRI2CAccess.NbByte>sizeof(TabByte)) { Result=-EINVAL; break; }

			if ((Result=verify_area(VERIFY_READ,ZRI2CAccess.TabByte,
						sizeof(u8)*ZRI2CAccess.NbByte))!=0) break;
			if ((Result=verify_area(VERIFY_WRITE,ZRI2CAccess.TabByte,
						sizeof(u8)*ZRI2CAccess.NbByte))!=0) break;
			memcpy_fromfs(TabByte,ZRI2CAccess.TabByte,ZRI2CAccess.NbByte*sizeof(u8));
			Result=I2CRead(pZoran,ZRI2CAccess.I2CAdr,ZRI2CAccess.I2CSubAdr,
					TabByte,ZRI2CAccess.NbByte);
			memcpy_tofs(ZRI2CAccess.TabByte,TabByte,ZRI2CAccess.NbByte*sizeof(u8));
			}
			ZRDBG(("ZORAN_I2C_READ2\n"));
			break;
		case ZORAN_I2C_WRITE:
			ZRDBG(("ZORAN_I2C_WRITE1\n"));
			{
			ZR_I2C_ACCESS			ZRI2CAccess;
			u8				TabByte[256];
			if ((Result=verify_area(VERIFY_READ,(ZR_I2C_ACCESS*)arg,
						sizeof(ZR_I2C_ACCESS)))!=0) break;
			memcpy_fromfs(&ZRI2CAccess,(ZR_I2C_ACCESS*)arg,sizeof(ZR_I2C_ACCESS));
			if (ZRI2CAccess.NbByte>sizeof(TabByte)) { Result=-EINVAL; break; }

			if ((Result=verify_area(VERIFY_READ,ZRI2CAccess.TabByte,
						sizeof(u8)*ZRI2CAccess.NbByte))!=0) break;
			memcpy_fromfs(TabByte,ZRI2CAccess.TabByte,ZRI2CAccess.NbByte*sizeof(u8));
			Result=I2CWrite(pZoran,ZRI2CAccess.I2CAdr,ZRI2CAccess.I2CSubAdr,
					TabByte,ZRI2CAccess.NbByte);
			}
			ZRDBG(("ZORAN_I2C_WRITE2\n"));
			break;

		case ZORAN_DUMP_REG:
			ZoranDumpReg(pZoran);
			break;
		default:
			ERR(("Zoran%d ioctl error: %i unhandled.\n",pZoran->IndexPCI,Command));
			Result=-ENOTTY;
			break;
		}
	return(Result);
}

